@@ -41,6 +41,7 @@ class OrcaNavListGUI:
         self._gui = self._createNavListDialog(columnHeaders, rows, selectedRow)
         self._gui.set_title(title)
         self._gui.set_modal(True)
+        self._script = orca_state.activeScript
         self.showGUI()
 
     def _createNavListDialog(self, columnHeaders, rows, selectedRow):
@@ -59,7 +60,7 @@ class OrcaNavListGUI:
         self._tree.set_vexpand(True)
         scrolledWindow.add(self._tree)
 
-        cols = [GObject.TYPE_OBJECT]
+        cols = [GObject.TYPE_OBJECT, GObject.TYPE_INT]
         cols.extend(len(columnHeaders) * [GObject.TYPE_STRING])
         model = Gtk.ListStore(*cols)
 
@@ -68,10 +69,15 @@ class OrcaNavListGUI:
         column.set_visible(False)
         self._tree.append_column(column)
 
+        cell = Gtk.CellRendererText()
+        column = Gtk.TreeViewColumn("offset", cell, text=1)
+        column.set_visible(False)
+        self._tree.append_column(column)
+
         for i, header in enumerate(columnHeaders):
             cell = Gtk.CellRendererText()
-            column = Gtk.TreeViewColumn(header, cell, text=i+1)
-            column.set_sort_column_id(i+1)
+            column = Gtk.TreeViewColumn(header, cell, text=i+2)
+            column.set_sort_column_id(i+2)
             self._tree.append_column(column)
 
         for row in rows:
@@ -90,7 +96,6 @@ class OrcaNavListGUI:
         btn.grab_default()
         btn.connect('clicked', self._onJumpToClicked)
 
-
         self._activateButton = dialog.add_button(
             guilabels.ACTIVATE, Gtk.ResponseType.OK)
         self._activateButton.connect('clicked', self._onActivateClicked)
@@ -108,7 +113,7 @@ class OrcaNavListGUI:
         self._gui.present_with_time(ts)
 
     def _onCursorChanged(self, widget):
-        obj = self._getSelectedAccessible()
+        obj, offset = self._getSelectedAccessibleAndOffset()
         try:
             action = obj.queryAction()
         except:
@@ -129,28 +134,12 @@ class OrcaNavListGUI:
         self._gui.destroy()
 
     def _onJumpToClicked(self, widget):
-        obj = self._getSelectedAccessible()
+        obj, offset = self._getSelectedAccessibleAndOffset()
         self._gui.destroy()
-        try:
-            obj.queryComponent().grabFocus()
-        except:
-            debug.println(debug.LEVEL_FINE, 'Could not grab focus on %s' % obj)
-        try:
-            text = obj.queryText()
-            text.setCaretOffset(0)
-        except NotImplementedError:
-            while obj.childCount:
-                obj = obj[0]
-                try:
-                    text = obj.queryText()
-                    text.setCaretOffset(0)
-                except NotImplementedError:
-                    pass
-                else:
-                    break
+        self._script.utilities.setCaretPosition(obj, offset)
 
     def _onActivateClicked(self, widget):
-        obj = self._getSelectedAccessible()
+        obj, offset = self._getSelectedAccessibleAndOffset()
         self._gui.destroy()
         try:
             action = obj.queryAction()
@@ -160,7 +149,7 @@ class OrcaNavListGUI:
         else:
             action.doAction(0)
 
-    def _getSelectedAccessible(self):
+    def _getSelectedAccessibleAndOffset(self):
         if not self._tree:
             return None
 
@@ -172,7 +161,9 @@ class OrcaNavListGUI:
         if not paths:
             return None
 
-        return model.get_value(model.get_iter(paths[0]), 0)
+        obj = model.get_value(model.get_iter(paths[0]), 0)
+        offset = model.get_value(model.get_iter(paths[0]), 1)
+        return obj, max(0, offset)
 
 def showUI(title='', columnHeaders=[], rows=[()], selectedRow=0):
     gui = OrcaNavListGUI(title, columnHeaders, rows, selectedRow)
