@@ -955,7 +955,7 @@ class Script(default.Script):
         """Handles changes of focus of interest to the script."""
 
         if newFocus and self.utilities.isZombie(newFocus):
-            msg = "WEB: New focus is Zombie" % newFocus
+            msg = "WEB: New focus is Zombie: %s" % newFocus
             debug.println(debug.LEVEL_INFO, msg)
             return True
 
@@ -988,6 +988,28 @@ class Script(default.Script):
 
         return True
 
+    def onActiveChanged(self, event):
+        """Callback for object:state-changed:active accessibility events."""
+
+        if not self.utilities.inDocumentContent(event.source):
+            msg = "WEB: Event source is not in document content"
+            debug.println(debug.LEVEL_INFO, msg)
+            return False
+
+        if not event.detail1:
+            msg = "WEB: Ignoring because event source is now inactive"
+            debug.println(debug.LEVEL_INFO, msg)
+            return True
+
+        role = event.source.getRole()
+        if role in [pyatspi.ROLE_DIALOG, pyatspi.ROLE_ALERT]:
+            msg = "WEB: Event handled: Setting locusOfFocus to event source"
+            debug.println(debug.LEVEL_INFO, msg)
+            orca.setLocusOfFocus(event, event.source)
+            return True
+
+        return False
+
     def onBusyChanged(self, event):
         """Callback for object:state-changed:busy accessibility events."""
 
@@ -1066,7 +1088,7 @@ class Script(default.Script):
         elif not _settingsManager.getSetting('sayAllOnLoad'):
             msg = "WEB: Not doing SayAll due to sayAllOnLoad being False"
             debug.println(debug.LEVEL_INFO, msg)
-            self.speakContents(self.getLineContentsAtOffset(obj, offset))
+            self.speakContents(self.utilities.getLineContentsAtOffset(obj, offset))
         elif _settingsManager.getSetting('enableSpeech'):
             msg = "WEB: Doing SayAll"
             debug.println(debug.LEVEL_INFO, msg)
@@ -1091,7 +1113,8 @@ class Script(default.Script):
             return False
 
         if not self.utilities.inDocumentContent(orca_state.locusOfFocus):
-            msg = "WEB: Ignoring event source is in document; locusOfFocus is not"
+            msg = "WEB: Event ignored: locusOfFocus (%s) is not in document content" \
+                  % orca_state.locusOfFocus
             debug.println(debug.LEVEL_INFO, msg)
             return True
 
@@ -1281,7 +1304,7 @@ class Script(default.Script):
             utterances.append(messages.NEW_ITEM_ADDED)
             utterances.extend(self.speechGenerator.generateSpeech(child, force=True))
             speech.speak(utterances)
-            self.lastMouseOverObject = child
+            self._lastMouseOverObject = child
             self.preMouseOverContext = self.utilities.getCaretContext()
             return True
 
@@ -1412,7 +1435,8 @@ class Script(default.Script):
             return False
 
         if not self.utilities.inDocumentContent(orca_state.locusOfFocus):
-            msg = "WEB: Ignoring event source is in document; locusOfFocus is not"
+            msg = "WEB: Event ignored: locusOfFocus (%s) is not in document content" \
+                  % orca_state.locusOfFocus
             debug.println(debug.LEVEL_INFO, msg)
             return True
 
@@ -1469,8 +1493,8 @@ class Script(default.Script):
 
         state = event.source.getState()
         if not state.contains(pyatspi.STATE_EDITABLE):
-            if self.inMouseOverObject \
-               and self.utilities.isZombie(self.lastMouseOverObject):
+            if self._inMouseOverObject \
+               and self.utilities.isZombie(self._lastMouseOverObject):
                 msg = "WEB: Restoring pre-mouseover context"
                 debug.println(debug.LEVEL_INFO, msg)
                 self.restorePreMouseOverContext()
@@ -1557,7 +1581,8 @@ class Script(default.Script):
             return True
 
         if not self.utilities.inDocumentContent(orca_state.locusOfFocus):
-            msg = "WEB: Ignoring: Event in document content; focus is not"
+            msg = "WEB: Event ignored: locusOfFocus (%s) is not in document content" \
+                  % orca_state.locusOfFocus
             debug.println(debug.LEVEL_INFO, msg)
             return True
 
