@@ -65,6 +65,7 @@ class Utilities(script_utilities.Utilities):
         self._isNonEntryTextWidget = {}
         self._isUselessImage = {}
         self._inferredLabels = {}
+        self._roleDescription = {}
         self._text = {}
         self._tag = {}
         self._treatAsDiv = {}
@@ -103,6 +104,7 @@ class Utilities(script_utilities.Utilities):
         self._isNonEntryTextWidget = {}
         self._isUselessImage = {}
         self._inferredLabels = {}
+        self._roleDescription = {}
         self._tag = {}
         self._treatAsDiv = {}
         self._cleanupContexts()
@@ -319,6 +321,20 @@ class Utilities(script_utilities.Utilities):
 
         return lastChild
 
+    def getRoleDescription(self, obj):
+        rv = self._roleDescription.get(hash(obj))
+        if rv is not None:
+            return rv
+
+        try:
+            attrs = dict([attr.split(':', 1) for attr in obj.getAttributes()])
+        except:
+            attrs = {}
+
+        rv = attrs.get('roledescription', '')
+        self._roleDescription[hash(obj)] = rv
+        return rv
+
     def _getTag(self, obj):
         rv = self._tag.get(hash(obj))
         if rv is not None:
@@ -806,6 +822,9 @@ class Utilities(script_utilities.Utilities):
                 math = self.getMathAncestor(obj)
             return [[math, 0, 1, '']]
 
+        if obj.getRole() == pyatspi.ROLE_INTERNAL_FRAME and obj.childCount == 1:
+            return self._getContentsForObj(obj[0], 0, boundary)
+
         string, start, end = self._getTextAtOffset(obj, offset, boundary)
         if not string:
             return [[obj, start, end, string]]
@@ -1738,6 +1757,68 @@ class Utilities(script_utilities.Utilities):
         self._isOffScreenLabel[hash(obj)] = rv
         return rv
 
+    def isDetachedDocument(self, obj):
+        docRoles = [pyatspi.ROLE_DOCUMENT_FRAME, pyatspi.ROLE_DOCUMENT_WEB]
+        if (obj and obj.getRole() in docRoles):
+            if obj.parent is None:
+                msg = "WEB: %s is a detatched document" % obj
+                debug.println(debug.LEVEL_INFO, msg)
+                return True
+
+        return False
+
+    def iframeForDetachedDocument(self, obj, root=None):
+        root = root or self.documentFrame()
+        isIframe = lambda x: x and x.getRole() == pyatspi.ROLE_INTERNAL_FRAME
+        try:
+            iframes = pyatspi.findAllDescendants(root, isIframe)
+        except:
+            msg = "WEB: Exception getting descendant iframes of %s" % root
+            debug.println(debug.LEVEL_INFO, msg)
+            return None
+
+        for iframe in iframes:
+            if obj in iframe:
+                # We won't change behavior, but we do want to log all bogosity.
+                self._isBrokenChildParentTree(obj, iframe)
+
+                msg = "WEB: Returning %s as iframe parent of detached %s" % (iframe, obj)
+                debug.println(debug.LEVEL_INFO, msg)
+                return iframe
+
+        return None
+
+    def _isBrokenChildParentTree(self, child, parent):
+        if not (child and parent):
+            return False
+
+        try:
+            childIsChildOfParent = child in parent
+        except:
+            msg = "WEB: Exception checking if %s is in %s" % (child, parent)
+            debug.println(debug.LEVEL_INFO, msg)
+            childIsChildOfParent = False
+        else:
+            msg = "WEB: %s is child of %s: %s" % (child, parent, childIsChildOfParent)
+            debug.println(debug.LEVEL_INFO, msg)
+
+        try:
+            parentIsParentOfChild = child.parent == parent
+        except:
+            msg = "WEB: Exception getting parent of %s" % child
+            debug.println(debug.LEVEL_INFO, msg)
+            parentIsParentOfChild = False
+        else:
+            msg = "WEB: %s is parent of %s: %s" % (parent, child, parentIsParentOfChild)
+            debug.println(debug.LEVEL_INFO, msg)
+
+        if parentIsParentOfChild != childIsChildOfParent:
+            msg = "FAIL: The above is broken and likely needs to be fixed by the toolkit."
+            debug.println(debug.LEVEL_INFO, msg)
+            return True
+
+        return False
+
     def isInferredLabelForContents(self, content, contents):
         obj, start, end, string = content
         objs = list(filter(self.shouldInferLabelFor, [x[0] for x in contents]))
@@ -2279,6 +2360,7 @@ class Utilities(script_utilities.Utilities):
             return None, -1
 
         lookInChild = [pyatspi.ROLE_LIST,
+                       pyatspi.ROLE_INTERNAL_FRAME,
                        pyatspi.ROLE_TABLE,
                        pyatspi.ROLE_TABLE_ROW]
         if role in lookInChild and obj.childCount:
@@ -2339,7 +2421,8 @@ class Utilities(script_utilities.Utilities):
                         return obj, i
             elif not self.doNotDescendForCaret(obj) and obj.childCount:
                 return self.findNextCaretInOrder(obj[0], -1)
-            elif offset < 0 and not self.isTextBlockElement(obj) and not self.hasNoSize(obj):
+            elif offset < 0 and not self.isTextBlockElement(obj) and not self.hasNoSize(obj) \
+                 and not self.isUselessImage(obj):
                 return obj, 0
 
         # If we're here, start looking up the the tree, up to the document.
@@ -2348,6 +2431,10 @@ class Utilities(script_utilities.Utilities):
             return None, -1
 
         while obj.parent:
+            if self.isDetachedDocument(obj.parent):
+                obj = self.iframeForDetachedDocument(obj.parent)
+                continue
+
             parent = obj.parent
             if self.isZombie(parent):
                 replicant = self.findReplicant(self.documentFrame(), parent)
@@ -2398,7 +2485,8 @@ class Utilities(script_utilities.Utilities):
                         return obj, i
             elif not self.doNotDescendForCaret(obj) and obj.childCount:
                 return self.findPreviousCaretInOrder(obj[obj.childCount - 1], -1)
-            elif offset < 0 and not self.isTextBlockElement(obj) and not self.hasNoSize(obj):
+            elif offset < 0 and not self.isTextBlockElement(obj) and not self.hasNoSize(obj) \
+                 and not self.isUselessImage(obj):
                 return obj, 0
 
         # If we're here, start looking up the the tree, up to the document.
@@ -2407,6 +2495,10 @@ class Utilities(script_utilities.Utilities):
             return None, -1
 
         while obj.parent:
+            if self.isDetachedDocument(obj.parent):
+                obj = self.iframeForDetachedDocument(obj.parent)
+                continue
+
             parent = obj.parent
             if self.isZombie(parent):
                 replicant = self.findReplicant(self.documentFrame(), parent)
